/*
 * Growable text buffers.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <stdlib.h>
#include "buf.h"

#define iswhite(c)   ((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\r')

/* Allocate storage for a text buffer */
void
buf_init(buf)
	buf_t *buf;
{
	buf->alloc = 80;
	buf->data = malloc(buf->alloc);
	buf->len = 0;
}

/* Append a single character */
void
buf_append(buf, c)
	buf_t *buf;
	char c;
{
	if (buf == NULL)
		putchar(c);
	else {
		buf->data[buf->len++] = c;
		/* ensure always 1 byte more alloc'd than len */
		while (buf->len + 1 > buf->alloc) 
			buf->data = realloc(buf->data, (buf->alloc *= 2));
	}
}

/* Append a C string */
void
buf_addstr(buf, cp)
	buf_t *buf;
	char *cp;
{
	while (*cp) 
		buf_append(buf, *cp++);
}

/* Append contents of another buffer */
void
buf_cat(buf1, buf2)
	buf_t *buf1, *buf2;
{
	if (buf1 == NULL) {
		fwrite(buf2->data, buf2->len, 1, stdout);
		return;
	}
	if (buf1->len + buf2->len + 1 > buf1->alloc) {
		while (buf1->len + buf2->len + 1 > buf1->alloc)
			buf1->alloc *= 2;
		buf1->data = realloc(buf1->data, buf1->alloc);
	}
	memcpy(buf1->data + buf1->len, buf2->data, buf2->len);
	buf1->len += buf2->len;
}

/* Release storage allocated by buf_init */
void
buf_free(buf)
	buf_t *buf;
{
	free(buf->data);
	buf->data = NULL;
	buf->alloc = buf->len = 0;
}
