/*
 * Character strings.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "value.h"
#include "chr.h"

/* Associate storage for a character string */
void
chr_realloc(c, len)
	struct chr *c;
	int len;
{
	if (len < 0)
		len = 0;
	if (c->len && len == 0)
		free(c->data);
	else if (c->len == 0 && len) {
		c->data = malloc(len + 1);
		c->data[len] = '\0';
	} else {
		c->data = realloc(c->data, len + 1);
		c->data[len] = '\0';
	}
	c->len = len;
	if (len == 0)
		c->data = "";
}

/* Set a chr string using a C string */
void
chr_setstr(c, cp)
	struct chr *c;
	const char *cp;
{
	int len = strlen(cp);
	chr_realloc(c, len);
	if (len)
		memcpy(c->data, cp, len);
}

/* Concatenate */
void
chr_cat(dst, a, b)
	struct chr *dst, *a, *b;
{
	chr_realloc(dst, a->len + b->len);
	memcpy(dst->data, a->data, a->len);
	memcpy(dst->data + a->len, b->data, b->len);
}

/* Duplicate */
void
chr_dup(dst, src)
	struct chr *dst, *src;
{
	chr_realloc(dst, src->len);
	memcpy(dst->data, src->data, dst->len);
}

/* Case-insensitive comparison */
int
chr_cmp(a, b)
	struct chr *a, *b;
{
	int ret, i;

	if (a->len < b->len)
		return -chr_cmp(b, a);
	ret = strncasecmp(a->data, b->data, b->len);
	if (ret != 0)
		return ret;
	for (i = b->len; i < a->len; i++) {
		if (a->data[i] == '\0')
			break;
		if (a->data[i] != ' ')
			return a->data[i];
	}
	return 0;
}

/* Substitute function */
void
chr_subst(r, f, _a, _na)
	struct chr *r, *f;
	value_t **_a;
	int _na;
{
	int rp, i;

#ifdef DEBUG
fprintf(stderr, "chr_subst: f='%.*s' _na=%d _a=[", f->len, f->data, _na);
for (i = 0; i < _na; i++) {
	if (i != 0) fprintf(stderr, ", ");
	fprintf(stderr, "'%.*s'", _a[i]->v_chr.len, _a[i]->v_chr.data);
}
fprintf(stderr, "]\n");
#endif

	/* Predict length of result string */
	for (rp = i = 0; i < f->len; i++)
		if (f->data[i] == '&' && i + 1 < f->len) {
			if (f->data[i+1] == '&') {
				i++;
				rp++;
			} else {
				int ref = 0;
				while (i+1 < f->len && 
				       f->data[i+1] >= '0' &&
				       f->data[i+1] <= '9')
					ref = (ref*10) + (f->data[++i]-'0');
				if (ref > 0 && ref <= _na) {
					rp += _a[ref - 1]->v_chr.len;
				}
			}
		} else
			rp++;

	/* Allocate storage */
	chr_realloc(r, rp);

	/* Copy format and referenced arguments */
	for (rp = i = 0; i < f->len; i++)
		if (f->data[i] == '&' && i + 1 < f->len) {
			if (f->data[i+1] == '&') {
				i++;
				r->data[rp++] = '&';
			} else {
				int ref = 0;
				while (i+1 < f->len && 
				       f->data[i+1] >= '0' &&
				       f->data[i+1] <= '9')
					ref = (ref*10) + (f->data[++i]-'0');
				if (ref > 0 && ref <= _na) {
					struct chr *t = &_a[ref - 1]->v_chr;
					if (t->len) {
						memcpy(r->data + rp,
						    t->data, t->len);
						rp += t->len;
					}
				}
			}
		} else
			r->data[rp++] = f->data[i];
}
