/*
 * Conditional expression lexical analyser.
 *
 * David Leonard, 2001. Public domain.
 */
%{
#include <string.h>
#include "value.h"
#include "y.tab.h"

#define YY_NO_UNPUT
#define yywrap() 1
void yyerror(const char *, ...);

%}

escape		[~\\]

%%

[0-9]+\.[0-9]*		{ yylval.value.v_int = atoi(yytext);	/* XXX */
			  yylval.value.v_type = INT;
			  return VALUE; }
[0-9]+			{ yylval.value.v_int = atoi(yytext);
			  yylval.value.v_type = INT;
			  return VALUE; }
"?"			{ yylval.value.v_type = UNK;
			  return VALUE; }
\'({escape}[0-3][0-7][0-7]|{escape}.|[^'])*\'	|
\"({escape}[0-3][0-7][0-7]|{escape}.|[^"])*\"	{
			  int i, j;
			  yylval.value.v_type = CHR;
			  if (yyleng == 2) {
				  yylval.value.v_chr.len = 0;
				  yylval.value.v_chr.data = "";
				  return VALUE;
			  }
			  yylval.value.v_chr.data = malloc(yyleng - 1);
			  for (i = 1, j = 0; i < yyleng - 1; )
			  {
				char c = yytext[i++];
				if (c == '~' || c == '\\') {
					c = yytext[i++];
					if (c == 'n') c = '\012';
					else if (c == 'r') c = '\015';
					else if (c == 't') c = '\011';
					else if (c >= '0' && c <= '3') {
						c =  ((c          -'0')&3)<<6;
						c |= ((yytext[i++]-'0')&7)<<3;
						c |= ((yytext[i++]-'0')&7);
					}
				} 
				yylval.value.v_chr.data[j++] = c;
			  }
			  yylval.value.v_chr.data[j] = '\0';
			  yylval.value.v_chr.len = j;
			  return VALUE; }

"&then"			{ return THEN; }
true			{ yylval.value.v_log = 1;
			  yylval.value.v_type = LOG;
			  return VALUE; }
false			{ yylval.value.v_log = 0;
			  yylval.value.v_type = LOG;
			  return VALUE; }
and			{ return AND; }
or			{ return OR; }
not			{ return NOT; }
begins			{ return BEGINS; }
matches			{ return MATCHES; }
defined			{ return DEFINED; }
[a-z][-a-z0-9]+		{ if (yyleng + 1 > sizeof yylval.ident)
			  	yyerror("identifier too long");
			  strncpy(yylval.ident, yytext, sizeof yylval.ident-1);
			  yylval.ident[sizeof yylval.ident-1] = '\0';
			  return IDENT;
			}

[ \t\n\r]		/* ignore */

"<="			{ return LE; }
">="			{ return GE; }
"<>"			{ return NE; }
[-+/*()<>=,]		{ return *yytext; }
.			{ yyerror("illegal character 0x%02x '%c'", 
			    yytext[0], yytext[0]); }

