/*
 * Function calling for conditional expression evaluation.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <string.h>
#include <err.h>
#include "value.h"
#include "func.h"
#include "keyword.h"

extern void yyerror(const char *, ...);

#ifdef DEBUG
void
printvalue(fp, v)
	FILE *fp;
	value_t *v;
{
	switch (v->v_type) {
	case INT:	fprintf(fp, "%d", v->v_int); break;
	case CHR:	fprintf(fp, "\"%.*s\"", v->v_chr.len, v->v_chr.data);
			break;
	case LOG:	fprintf(fp, "%s", v->v_log?"true":"false"); break;
	default:
	case UNK:	fprintf(fp, "?"); break;
	}
}

void
printtype(fp, t)
	FILE *fp;
	enum type t;
{
	switch (t) {
	case INT:	fprintf(fp, "<INT>"); break;
	case CHR:	fprintf(fp, "<CHR>"); break;
	case LOG:	fprintf(fp, "<LOG>"); break;
	default:
	case UNK:	fprintf(fp, "<UNK>"); break;
	}
}
#endif

void
notimp(srcfile, srcline, name)
	char *srcfile;
	int srcline;
	char *name;
{
	yyerror("'%s' not implemented [%s:%d]", name, srcfile, srcline);
}

void
apply(name, dest, arg, narg)
	char *name;
	value_t *dest;
	value_t **arg;
	int narg;
{
	int i, j;
	int namefound = 0;
	int hasunk = 0;
	struct functab *f = NULL;
	int wrongarg = -1;

	for (j = 0; j < narg; j++)
		if (arg[j]->v_type == UNK) {
			hasunk = 1;
			break;
		}

#ifdef DEBUG
fprintf(stderr, "Matching: %s(", name);
for (j = 0; j < narg; j++) {
	if (j != 0) fprintf(stderr, ", ");
	printtype(stderr, arg[j]->v_type);
	printvalue(stderr, arg[j]);
}
fprintf(stderr, ")\n");
#endif

	/* Look for an exact match */
	for (i = 0; i < nfunctab; i++) {
		if (keyword_match(name, functab[i].name) != 0)
			continue;
		namefound = 1;
		if (narg < functab[i].minargs)
			continue;
		if (functab[i].maxargs != -1 && narg > functab[i].maxargs)
			continue;
#ifdef DEBUG
fprintf(stderr, "Found:\t");
printtype(stderr, functab[i].ret);
fprintf(stderr, " %s(", functab[i].name);
{ int q; for (q = 0; q < functab[i].minargs; q++) {
	if (q != 0) fprintf(stderr, ", ");
	printtype(stderr, functab[i].type[q]); }
}
if (functab[i].maxargs == -1) 
	fprintf(stderr, "...");
fprintf(stderr, ")\n");
#endif
		for (j = 0; j < narg; j++) {
			enum type atype = arg[j]->v_type;
			enum type ftype = 
				(functab[i].maxargs == -1 &&
				 j >= functab[i].minargs)
				 ? functab[i].type[functab[i].minargs - 1]
				 : functab[i].type[j];
			if (ftype == ANY || ftype == atype)
				continue;
			break;
		}

		if (j == narg) {
			f = &functab[i];
			break;
		} else if (j > wrongarg)
			wrongarg = j;
	}

	/* Couldn't find a name even? */
	if (!namefound) {
		yyerror("unknown function '%s'", name);
		dest->v_type = UNK;
		return;
	}

	/* If no exact match, and has an unknown, default to unknown */
	if (hasunk) {
		dest->v_type = UNK;
		return;
	}

	if (f == NULL) {
		yyerror("wrong type for argument %d to %s()", wrongarg, name);
		dest->v_type = UNK;
		return;
	}

#ifdef DEBUG
fprintf(stderr, "Calling: %s(", functab[i].name);
for (j = 0; j < narg; j++) {
	if (j != 0) fprintf(stderr, ", ");
	printvalue(stderr, arg[j]);
}
fprintf(stderr, ")\n");
#endif

	(*functab[i].func)(dest, arg, narg);

#ifdef DEBUG
fprintf(stderr, "  -> ");
printvalue(stderr, dest);
fprintf(stderr, "\n");
#endif

}
