/*
 * Simple string->string hash table
 *
 * David Leonard, 2001. Public domain.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <ctype.h>
#include "hash.h"

#define HASHMAX 257

struct hashval {
	char *key;
	char *value;
	struct hashval *next;
};

static unsigned int
hash(key)
	char *key;
{
	unsigned int hash;

	for (hash = 0; *key; key++)
		hash = (hash << 1) ^ tolower(*key);
	return hash % 257;
}

struct hashval *
new_hashval(key, value)
	char *key, *value;
{
	struct hashval *hv = (struct hashval *)malloc(sizeof (struct hashval));

	hv->key = strdup(key);
	hv->value = strdup(value);
	return hv;
}

static void
free_hashval(hvp)
	struct hashval **hvp;
{
	struct hashval *hv = *hvp;

	*hvp = hv->next;
	free(hv->key);
	free(hv->value);
	free(hv);
}

static struct hashval **
hashval_find(hvp, key)
	struct hashval **hvp;
	char *key;
{
	while (*hvp != NULL && strcasecmp(key, (*hvp)->key) != 0)
		hvp = &(*hvp)->next;
	return hvp;
}

int
hash_put(h, key, value)
	hash_t *h;
	char *key, *value;
{
	int i = hash(key);
	struct hashval **hvp = hashval_find(&h->hashtab[i], key);
	struct hashval *hv = *hvp;

	if (hv != NULL) {
		free(hv->value);
		hv->value = strdup(value);
		return 1;
	}

	hv = new_hashval(key, value);
	hv->next = h->hashtab[i];
	h->hashtab[i] = hv;
	return 0;
}

char *
hash_get(h, key)
	hash_t *h;
	char *key;
{
	int i = hash(key);
	struct hashval **hvp = hashval_find(&h->hashtab[i], key);
	struct hashval *hv = *hvp;
	
	if (hv == NULL)
		return NULL;
	else
		return hv->value;
}

void
hash_del(h, key)
	hash_t *h;
	char *key;
{
	int i = hash(key);
	struct hashval **hvp = hashval_find(&h->hashtab[i], key);

	if (*hvp != NULL)
		free_hashval(hvp);
}

hash_t *
new_hash()
{
	hash_t *h;
	int i;

	h = (hash_t *)malloc(sizeof (hash_t));
	if (h == NULL)
		errx(1, "malloc");
	h->hashtab = (struct hashval **)malloc(HASHMAX * sizeof (struct hashval *));
	if (h->hashtab == NULL)
		errx(1, "malloc");
	for (i = 0; i < HASHMAX; i++)
		h->hashtab[i] = NULL;
	return h;
}

void
free_hash(h)
	hash_t *h;
{
	int i;

	for (i = 0; i < HASHMAX; i++)
		while (h->hashtab[i] != NULL)
			free_hashval(&h->hashtab[i]);
	free(h->hashtab);
	free(h);
}
