/*
 * If-then-else state machine.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <err.h>
#include <stdlib.h>
#include "ifelse.h"

struct ifelse {
	int nowactive;
	int elseactive;
};

static struct ifelse *stackbase;
static struct ifelse *stacktop;
static int stackalloc;

static void
push()
{
	if (stackbase == NULL) {
		stackalloc = 16;
		stackbase = stacktop = (struct ifelse *)malloc(stackalloc * 
			sizeof (struct ifelse));
	} else
		stacktop++;
	if (stacktop - stackbase >= stackalloc) {
		int relativepos = stacktop - stackbase;
		stackalloc *= 2;
		stackbase = (struct ifelse *)realloc(stackbase,
			stackalloc * sizeof (struct ifelse));
		stacktop = stackbase + relativepos;
	}
}

static void
pop()
{
	stacktop--;
	if (stacktop < stackbase)
		errx(1, "ifelse stack underflow");
}


void
if_init()
{
	push();
	stacktop->nowactive = 1;
	stacktop->elseactive = 1;
}

int
if_active()
{
	return stacktop->nowactive;
}

int
if_elseactive()
{
	return stacktop->elseactive && (stacktop - 1)->nowactive;
}

void
if_if(condition)
	int condition;
{
	int wasactive = stacktop->nowactive;
	push(); 
	stacktop->nowactive = wasactive && condition;
	stacktop->elseactive = !condition;
}

void
if_else(condition)
	int condition;
{
	int outeractive = (stacktop-1)->nowactive;
	int elsewasactive = stacktop->elseactive;

	stacktop->nowactive = outeractive && elsewasactive && condition;
	stacktop->elseactive = elsewasactive && !condition;
}

void
if_endif()
{
	pop();
}
