/*
 * Evaluate string as logical expression using the expression parser.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <err.h>
#include <stdlib.h>
#include "buf.h"
#include "hash.h"
#include "scope.h"
#include "value.h"
#include "ifexpr.h"
#include "y.tab.h"

/* Interface to code generated by yacc/lex */
typedef void *YY_BUFFER_STATE;	/* XXX */
YY_BUFFER_STATE yy_scan_bytes(const char *, int);
void yy_delete_buffer(YY_BUFFER_STATE);
int yyparse(void);
int yylex(void);

/* Interface to results of parsing */
extern value_t parse_result;
extern int parse_errors;
extern scope_t *parse_scope;

int
ifexpr(scope, buf, start)
	scope_t *scope;
	struct buf *buf;
	int start;
{
	int failed;
	int token;
	int result;
	YY_BUFFER_STATE state;


	/* Force last two bytes of buffer to NUL */
	buf_append(buf, '\0');
	buf->len--;

#ifdef DEBUG
fprintf(stderr, "evaluate: '%.*s'\n", buf->len - start, buf->data + start);
#endif

	/* Look for the terminating &THEN */
	state = yy_scan_bytes(buf->data + start, buf->len - start);
	do {
		token = yylex();
		if (token == 0) {
			yy_delete_buffer(state);
			return -1;	/* ask for more data */
		}
	} while (token != THEN);
	yy_delete_buffer(state);

	/* Parse expression */
	state = yy_scan_bytes(buf->data + start, buf->len - start);
	parse_scope = scope;
	parse_errors = 0;
	parse_result.v_type = UNK;
	failed = yyparse();
	yy_delete_buffer(state);

	if (failed || parse_errors) {
		/* Treat parse errors as false conditional */
		result = 0;
	} else
		switch (parse_result.v_type) {
		case INT:
			result = parse_result.v_int != 0;
			break;
		case CHR:
			result = parse_result.v_chr.len != 0;
			break;
		case LOG:
			result = parse_result.v_log;
			break;
		case UNK:
		default:
#ifdef DEBUG
			warnx("%s:%d: invalid condition result",
				scope->filename, scope->lineno);
#endif
			result = 0;
		}

	value_free(parse_result);
	return result;
}
