/*
 * Keyword matching
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "keyword.h"

/* The list of Progress keywords and reserved keywords */
#define RESERVED	1
static struct keyword {
	char	*pattern;
	int	flags;
} keywords[] = {
#include "keywordlist.inc"
};
static int nkeywords = sizeof keywords/sizeof keywords[0];

static int
compar(a, b)
	void *a, *b;
{
	char *word = (char *)a;
	struct keyword *kb = (struct keyword *)b;
	return keyword_match(word, kb->pattern);
}

char *
keyword(s, all)
	char *s;
	int all;
{
	struct keyword *res;
	int i;
	static char tmp[80];	/* no keyword is longer than this */

	res = (struct keyword *)bsearch(s, keywords, nkeywords,
	    sizeof keywords[0], compar);

	if (res == NULL || (res->flags != RESERVED && all))
		return NULL;

	/* Copy just the non-optional part of the pattern */
	for (i = 0; res->pattern[i] && res->pattern[i] != '|'; i++)
		tmp[i] = res->pattern[i];
	tmp[i] = '\0';
	return tmp;
}

/*
 * Match a keyword string with a keyword pattern. A | character in the
 * pattern indicates that future characters are optional.
 * e.g   keyword_match("foob", "foo|bar") will return 0
 */
int
keyword_match(s, pattern)
	char *s, *pattern;
{
	int inopt = 0;
	int ret;

	while (1) {
		if (*pattern == '|') {
			pattern++;
			inopt = 1;
			continue;
		}
		if (*s == '\0' && (inopt || *pattern == '\0'))
			return 0;
		ret =  tolower(*pattern) - tolower(*s); 
		pattern++; s++;
		if (ret != 0)
			return ret;
	}
}

