/*
 * Matches operator.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <err.h>
#include <regex.h>

/* Characters that need to be escaped in a regex */
static char metachars[] = ".*[^$|+?\\";

int
matches(s, pattern, begins)
	char *s, *pattern;
	int begins;
{
	char *restr = malloc(strlen(pattern) * 2 + 3);
	char *p, *r;
	regex_t re;
	char buf[256];
	int error;

	r = restr;
	*r++ = '^';
	for (p = pattern; *p; p++) {
		if      (*p == '.') { *r++ = '.'; }
		else if (*p == '*') { *r++ = '.'; *r++ = '*'; }
		else if (*p == '~' || *p == '\\') {
			p++;
			if (strchr(metachars, *p)) *r++ = '\\';
			*r++ = *p;
		} else {
			if (strchr(metachars, *p)) *r++ = '\\';
			*r++ = *p;
		}
	}
	if (!begins)
		*r++ = '$';
	*r = '\0';

#ifdef DEBUG
fprintf(stderr, "pattern '%s' = regex '%s'\n", pattern, restr);
#endif

	error = regcomp(&re, restr, REG_BASIC | REG_ICASE | REG_NOSUB);
	free(restr);
	if (error != 0) {
		regerror(error, &re, buf, sizeof buf);
		errx(1, "regcomp: %s", buf);
	}
	error = regexec(&re, s, 0, NULL, 0);
	regfree(&re);

#ifdef DEBUG
fprintf(stderr, "exec result: %d\n", error);
#endif

	if (error == REG_NOMATCH) return 0;
	if (error == 0)	return 1;

	regerror(error, &re, buf, sizeof buf);
	errx(1, "regcomp: %s", buf);
}
