/*
 * Tokenize words in a braced include expansion.
 *
 * David Leonard, 2001. Public domain.
 */

#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include "hash.h"
#include "buf.h"
#include "scope.h"
#include "tokenize.h"

#define iswhite(c)   ((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\r')

char **
tokenize(scope, buf)
	scope_t *scope;
	buf_t *buf;
{
	char **args;
	int i;
	int nargs = 0;
	int nalloc = 8;

	args = (char **)malloc(nalloc * sizeof(char *));
	i = 0;
	/* skip leading white */
	while (i < buf->len && iswhite(buf->data[i])) i++;

	while (i < buf->len) {
		char *p;
		int inquote = 0;
		int ampstart = 0;

		/* mark beginning of word */
		p = args[nargs] = buf->data + i;

		/* Check for an ampersand start */
		if (i < buf->len && buf->data[i] == '&') {
			i++; p++;
			ampstart = 1;
		}

		while (i < buf->len) {
#ifdef DEBUG
fprintf(stderr, "inquote=%d ampstart=%d buf->data[i]='%c'\n",
			inquote, ampstart, buf->data[i]);
#endif
			if (!inquote && buf->data[i] == '"') {
				i++;
				if (buf->data[i] != '"') {
					inquote = 1;
					continue;
				}
			}
			if (inquote && buf->data[i] == '"') {
				i++;
				if (buf->data[i] != '"') {
					inquote = 0;
					continue;
				}
			}
			if (!inquote && iswhite(buf->data[i])) {
				if (ampstart == 0)
					break;
				else if (ampstart == 1)	/* found amp */
					ampstart = 2;	/* space after name */
				else if (ampstart == 3)	/* seen = */
					ampstart = 4;	/* space after = */
				if (ampstart == 2 || ampstart == 4) {
					i++;
					continue;
				}
			} else {
				if (!inquote && (ampstart == 1 || ampstart == 2)
				    && buf->data[i] == '=')
					ampstart = 3;
				else if (ampstart == 3 || ampstart == 4)
					ampstart = 0;
			}
			*p++ = buf->data[i++];
		}
		if (inquote)
			warnx("unterminated \"");
		/* Terminate */
		*p = '\0';
#ifdef DEBUG
fprintf(stderr, "result='%s'\n", args[nargs]);
#endif
		if (i < buf->len) i++;
		/* Skip whitespace after word */
		while (i < buf->len && iswhite(buf->data[i])) i++;
		/* Count the words */
		nargs++;
		while (nargs >= nalloc)
			args = (char **)realloc(args, 
				(nalloc *= 2) * sizeof(char *));
	}
	args[nargs] = NULL;
	return args;
}
